#  Copyright (C) 2026
#  ABM, Moscow
#
#  UNPUBLISHED PROPRIETARY MATERIAL.
#  ALL RIGHTS RESERVED.
#
#  Authors: Mike Orlov <m.orlov@abm-jsc.ru>
from typing import TypeVar, MutableMapping, Callable, Any

from typing_extensions import deprecated

from .is_none import is_not_none


T = TypeVar('T')
Condition = Callable[[T], bool]


def conditional_setitem(mapping: MutableMapping[str, Any], *, _condition: Condition = is_not_none, **kwargs: T) -> None:
    for key, value in kwargs.items():
        if _condition(value):
            mapping[key] = value


@deprecated('Use conditional_setitem instead')
def conditional_set(mapping: MutableMapping[str, Any], *, _condition: Condition = is_not_none, **kwargs: T) -> None:
    return conditional_setitem(mapping, _condition=_condition, **kwargs)



#  Copyright (C) 2026
#  ABM, Moscow
#
#  UNPUBLISHED PROPRIETARY MATERIAL.
#  ALL RIGHTS RESERVED.
#
#  Authors: Mike Orlov <m.orlov@abm-jsc.ru>
from .write_to_dict_or_raise import write_to_dict_or_raise
from .update_dict_recursive import update_dict_recursive



#  Copyright (C) 2026
#  ABM, Moscow
#
#  UNPUBLISHED PROPRIETARY MATERIAL.
#  ALL RIGHTS RESERVED.
#
#  Authors: Mike Orlov <m.orlov@abm-jsc.ru>
import collections.abc
from dataclasses import dataclass


@dataclass(slots=True, frozen=True)
class StructureHint:
    fields: collections.abc.Sequence['FieldHint | StructureHint']
    name: str | None = None

    def __post_init__(self):
        object.__setattr__(self, 'fields', tuple(self.fields))

    def __repr__(self) -> str:
        parts = [] if self.name is None else [f'name={self.name!r}']
        parts += [f'fields={list(self.fields)}']
        return f'{type(self).__name__}({", ".join(parts)})'

    __str__ = __repr__



#  Copyright (C) 2024
#  ABM, Moscow
#
#  UNPUBLISHED PROPRIETARY MATERIAL.
#  ALL RIGHTS RESERVED.
#
#  Authors: Mike Orlov <m.orlov@abm-jsc.ru>
import argparse
from typing import Iterable

DEFAULT_KEY_VALUE_SEPARATOR = '='
DEFAULT_KWARG_FLAG_NAME = '-v'


def parse_args_as_file_with_kwargs(
        args: Iterable[str] | None = None,  # None means sys.argv[1:]
        kwarg_flag_name: str | None = None,  # None or empty str means default
        key_value_separator: str | None = None,  # None or empty str means default
) -> tuple[str | None, dict[str, str]]:
    """
    Used to parse cmd args
    >>> parse_args_as_file_with_kwargs(args=['../some.ini', '-v', 'key=42', '-v', 'foo=bar'])
    ('../some.ini', {'key': '42', 'foo': 'bar'})
    >>> parse_args_as_file_with_kwargs(args=[])
    (None, {})
    """
    kwarg_flag_name = kwarg_flag_name or DEFAULT_KWARG_FLAG_NAME
    key_value_separator = key_value_separator or DEFAULT_KEY_VALUE_SEPARATOR
    parser = argparse.ArgumentParser()
    parser.add_argument('config_file_path', nargs='?', default=None)
    overload_arg = parser.add_argument(kwarg_flag_name, action='append')
    parsed = parser.parse_args(args)
    raw_overload = getattr(parsed, overload_arg.dest) or []
    return parsed.config_file_path, dict([val.split(key_value_separator) for val in raw_overload])



#  Copyright (C) 2026
#  ABM, Moscow
#
#  UNPUBLISHED PROPRIETARY MATERIAL.
#  ALL RIGHTS RESERVED.
#
#  Authors: Mike Orlov <m.orlov@abm-jsc.ru>
from ._alias import JsonLoaded, Jsonable, CustomJsonable
from .custom_dumps import EnhancedJSONEncoder, custom_dumps
from .json_dumper import JsonDumper
from .read_json_file_by_path import read_json_file_by_path
from .repr_in_dumps import ReprInDumps



#  Copyright (C) 2026
#  ABM, Moscow
#
#  UNPUBLISHED PROPRIETARY MATERIAL.
#  ALL RIGHTS RESERVED.
#
#  Authors: Mike Orlov <m.orlov@abm-jsc.ru>
from typing import Protocol, Callable, Any

from ._alias import Jsonable, CustomJsonable


class JsonDumper(Protocol):
    @staticmethod
    def __call__(
            obj: CustomJsonable, *, skipkeys: bool = False, ensure_ascii: bool = True, check_circular: bool = True,
            allow_nan: bool = True, indent: int | None = None, separators: tuple[str, str] | None = None,
            default: Callable[[Any], Jsonable] = None, sort_keys: bool = False, **kw
    ) -> str:
        pass



#  Copyright (C) 2026
#  ABM, Moscow
#
#  UNPUBLISHED PROPRIETARY MATERIAL.
#  ALL RIGHTS RESERVED.
#
#  Authors: Mike Orlov <m.orlov@abm-jsc.ru>
from .init_logs import LogsConfig, CustomFormatter, init_logs
from .init_random import init_random
from .parse_args import Arg, parse_args
from .parse_args_as_dataclass import parse_args_as_dataclass
from .parse_args_as_dataclass_or_exit import parse_args_as_dataclass_or_exit
from .parse_args_as_file_with_kwargs import parse_args_as_file_with_kwargs
from .read_file_as_dict import read_file_as_dict, read_config



#  Copyright (C) 2026
#  ABM, Moscow
#
#  UNPUBLISHED PROPRIETARY MATERIAL.
#  ALL RIGHTS RESERVED.
#
#  Authors: Mike Orlov <m.orlov@abm-jsc.ru>
from typing import MutableMapping, Any

from typing_extensions import deprecated


def setitem_if(condition: bool, mapping: MutableMapping[str, Any], **kwargs: Any) -> None:
    """
    if condition is True, set all items in mapping
    >>> setitem_if(True, d := {'q': 1}, w=2, e=3)
    >>> d
    {'q': 1, 'w': 2, 'e': 3}
    >>> setitem_if(False, d := {'q': 1}, w=2, e=3)
    >>> d
    {'q': 1}
    """
    if condition:
        for key, value in kwargs.items():
            mapping[key] = value


@deprecated('Use setitem_if instead')
def set_if(condition: bool, mapping: MutableMapping[str, Any], **kwargs: Any) -> None:
    return setitem_if(condition=condition, mapping=mapping, **kwargs)



#  Copyright (C) 2026
#  ABM, Moscow
#
#  UNPUBLISHED PROPRIETARY MATERIAL.
#  ALL RIGHTS RESERVED.
#
#  Authors: Mike Orlov <m.orlov@abm-jsc.ru>
import dataclasses
import types
from dataclasses import dataclass
from typing import Any


@dataclass(slots=True, frozen=True)
class FieldHint:
    name: str
    type: type | types.UnionType | dataclasses.InitVar  # | typing.Union
    default: Any = dataclasses.field(default_factory=lambda: dataclasses.MISSING)

    def __eq__(self, other: 'FieldHint') -> bool:
        """
        InitVar[str] != InitVar[str], so distinct type check is required
        """
        if (not isinstance(other, FieldHint) or self.name != other.name or
                type(self.type) != type(self.type) or self.default != other.default):
            return False
        if isinstance(self.type, dataclasses.InitVar):
            return self.type.type == other.type.type
        return self.type == other.type

    def __repr__(self) -> str:
        parts = [f'name={self.name!r}'] + [f'type={self.type.__name__ if isinstance(self.type, type) else self.type}']
        parts += [] if self.default is dataclasses.MISSING else [f'default={self.default!r}']
        return f'{type(self).__name__}({", ".join(parts)})'

    __str__ = __repr__



#  Copyright (C) 2026
#  ABM, Moscow
#
#  UNPUBLISHED PROPRIETARY MATERIAL.
#  ALL RIGHTS RESERVED.
#
#  Authors: Mike Orlov <m.orlov@abm-jsc.ru>
from .add_if import add_if
from .append_if import append_if
from .conditional_add import conditional_add
from .conditional_append import conditional_append
from .conditional_setitem import conditional_setitem, conditional_set
from .is_none import is_none, is_not_none
from .setitem_if import setitem_if, set_if
